<?php
ini_set('memory_limit', '128M');

/*

File: ti-mobile-pull.php

The purpose of this script is to take Appcelerator's SDK from
their build cloud and place it into an SVN repository that
exists on disk.

This essentially will 
1. Remove all prior .svn directories from their build 
(I don't know why their build doesn't remove it).

2. Perform a 'svn add' if necessary and add files manually
that SVN doesn't seem to recognize?

3. SVN commit to repo.

*/

function get_json_from_url($url)
{
   $data = file_get_contents($url);
   $data = json_decode($data, $assoc=true);
   return $data;
}

function get_latest($array, $platform)
{  
   for ($i = count($array)-1; $i >= 0; $i--)
      if (strstr($array[$i]['filename'], $platform))
         return $array[$i];
}

function get_build($filename, $url)
{
//   $url = $url . $filename;

   $fp = fopen('../ti-mobile-sdk/tmp/' . $filename, 'w+');
   
   $ch = curl_init($url);
//   curl_setopt($ch, CURLOPT_TIMEOUT, 50);
   curl_setopt($ch, CURLOPT_FILE, $fp);
   curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
   curl_exec($ch);
   curl_close($ch);
   
   fclose($fp);
}

function get_dir_list($dir)
{
   $d = opendir($dir);
   while ($entry = readdir($d))
   {   
      if ($entry != "." && $entry != "..")
         $dir_list[] = $entry;
   }

   sort($dir_list);
   return $dir_list;
}

function extract_file($filename, $repo)
{
   exec('unzip -o ../ti-mobile-sdk/tmp/' . $filename . ' -d ../ti-mobile-sdk/tmp/');

   // we now have extraction, but we need to properly clean it
   $dir_list[] = get_dir_list('../ti-mobile-sdk/tmp/mobilesdk/');
   $dir_list[] = get_dir_list('../ti-mobile-sdk/tmp/mobilesdk/' . $dir_list[0][0]);
   
   $base_sdk_dir = '../ti-mobile-sdk/tmp/mobilesdk/' . $dir_list[0][0] . '/' . $dir_list[1][0] . '/';

   // this will clean up existing .svn directories
   //exec('find ' . $base_sdk_dir . ' -name .svn -exec \'rm -rf {}\\;\'');
   exec('rm -rf `find ' . $base_sdk_dir . ' -name .svn`');

   // making appropriate directories
   mkdir('../ti-mobile-sdk/' . $repo . '/' . $dir_list[1][0] . '/');

   // moving cleaned and extracted files to destination
   exec('cp -arv ' . $base_sdk_dir . '* ../ti-mobile-sdk/' . $repo . '/' . $dir_list[1][0] . '/');

   return $dir_list[1][0];
}

function exec_svn($dir_name, $revision, $rename = '')
{
   $dir = '../ti-mobile-sdk/svn/' . $dir_name . '/';

   if ($rename !== '')
   {
      $new_dir = '../ti-mobile-sdk/svn/' . $rename . '/';
      rename($dir, $new_dir);
      $dir = $new_dir;
   }

   // svn doesn't recognize these specific files sometimes
   // we'll add if not already added
   exec('svn add -q ' . $dir . '; svn add -q ' . $dir . 'iphone/*; svn add -q ' . $dir . 'iphone/libTiCore.a; svn add -q ' . $dir . 'iphone/libtiverify.a; svn add -q ' . dir . 'iphone/Classes/*');

   exec('svn ci -q ' . $dir . ' -m \'[auto-commit] sdk version: ' . $dir_name . ' git revision: ' . $revision . '\'');
}

function exec_git($dir_name, $revision, $rename = '')
{
   $dir = '../ti-mobile-sdk/git/' . $dir_name . '/';
   
   if ($rename !== '')
   {
      $new_dir = '../ti-mobile-sdk/git/' . $rename . '';
      rename($dir, $new_dir);
      $dir = $new_dir;
   }

   exec ('cd ' . $dir . '; git add .; git commit -m \'sdk update: ' . $revision . '\'; git push origin master');
}

$base_url = 'http://builds.appcelerator.com.s3.amazonaws.com/mobile/';

echo "Grabbing branches...\n";
$branch_data = get_json_from_url($base_url . 'branches.json');
print_r($branch_data);

$repo = 'svn';

foreach ($branch_data['branches'] as $branch)
{
   echo "Getting build list for " . $branch . "...\n";
   $builds = get_json_from_url($base_url . $branch . '/index.json');

   echo "Determining latest build for " . $branch . "...\n";
   $latest_build = get_latest($builds, 'osx');

   echo "Grabbing build " . $latest_build['git_revision'] . "...\n";
   get_build($latest_build['filename'], $base_url . $branch . '/' . $latest_build['filename']);

   echo "Extracting...\n";
   $dir_name = extract_file($latest_build['filename'], $repo, $latest_build['filename']);

   if (version_compare($dir_name, '1.6.0') <= 0)
      continue;
   
   echo "Committing...\n";
   
   if ($repo == 'git')
      exec_git($dir_name, $dir_name);
   else
      exec_svn($dir_name, $dir_name, $branch);

   echo "Cleaning up...\n";
   exec('rm -rf ../ti-mobile-sdk/tmp/*');
}

// this part will always grab the most stable package
get_build('stable.zip', 'http://api.appcelerator.net/p/v1/release-download?token=24rRGHk0');
$dir_name = extract_file('stable.zip', $repo);

if ($repo == 'git')
   exec_git($dir_name, 'stable', 'stable');
else
   exec_svn($dir_name, 'stable', 'stable');

exec('rm -rf ../ti-mobile-sdk/tmp/*');
?>
